/*
 * File: scanner.c
 * ---------------
 * This file implements the scanner.h interface.
 * taken from Robert's scanner.* modified sometime
 * in my cs21 class, and now beaten into ANSI quality.
 */

#include <stdio.h>
#include <ctype.h>
#include <strlib.h>
#include "scanner.h"


/*
 * Private variables
 * -----------------
 * buffer  -- Private copy of the char * passed to InitScanner
 * retval  -- a temp char* for holding return value
 * buflen  -- Length of the buffer, saved for efficiency
 * cpos    -- Current character position in the buffer
 * IgnoreSpaceFlag -- intean "switch" to for whether or not to return
 *                  tokens consisting of whitespace.  Default is FALSE.
 */


static char* buffer;
static char* retval;
static int buflen;
static int cpos;
static int IgnoreSpaceFlag;



/*
 * Function: InitScanner
 * ---------------------
 * All this function has to do is initialize the private
 * variables used in the package.
 */

void InitScanner(char * line)
{
    buffer = line;
    buflen = strlen(buffer);
    cpos = 0;
    IgnoreSpaceFlag = FALSE;
}

/*
 * Function: GetNextToken
 * ----------------------
 * The implementation of GetNextToken follows its behavioral
 * description as given in the interface: if the next character
 * is alphanumeric (i.e., a letter or digit), the function
 * searches to find an unbroken char * of such characters and
 * returns the entire char *.  If the current character is not
 * a letter or digit, a one-character char * containing that
 * character is returned.
 */

char * GetNextToken(void)
{
    char ch;
    int start;

    if (cpos >= buflen) {perror("No more tokens"); return NULL;}
    
    ch = buffer[cpos];
    
    if (isalnum(ch))
      {
        start = cpos;
        while (cpos < buflen && isalnum(buffer[cpos]))
	  {
            cpos++;
	  }
		retval=(char*)malloc(sizeof(char)*(cpos-start+1));
		snprintf(retval, (cpos-start+1), "%s", buffer+(start*sizeof(char)));
	  return (retval);
      }
    else
      {
	if((IgnoreSpaceFlag) && isspace(buffer[cpos]))
	  {
	    cpos++;
	    return GetNextToken();
	  }
	else
	  {
	    cpos++;
			retval=(char*)malloc(sizeof(char)+1);
			retval[0]=ch;
			retval[1]='\0';
			return retval;
	  }
      }
}

/*
 * Function: AtEndOfLine
 * ---------------------
 * This implementation compares the current buffer position
 * against the saved length.
 */

int AtEndOfLine(void)
{
    return (cpos >= buflen);
}


/*
 * Function: IgnoreSpace
 * ------------------------------
 * this function is very self explanatory.  If you still
 * don't understand, get out of the .c file and just read
 * interface.
 */

void IgnoreSpace(int value)
{
  if(value)
    IgnoreSpaceFlag = TRUE;
}


